@@ -173,7 +173,7 @@ class FeatureDB(object):
             raise helpers.FeatureNotFoundError(key)
         return self._feature_returner(**results)
 
-    def count_features_of_type(self, featuretype):
+    def count_features_of_type(self, featuretype=None):
         """
         Simple count of features.
 
@@ -185,7 +185,8 @@ class FeatureDB(object):
 
         `featuretype` : string
 
-            Feature type (e.g., "gene") to count.
+            Feature type (e.g., "gene") to count.  If None, then count *all*
+            features in the database.
 
         Returns
         -------
@@ -193,11 +194,18 @@ class FeatureDB(object):
 
         """
         c = self.conn.cursor()
-        c.execute(
-            '''
-            SELECT count() FROM features
-            WHERE featuretype = ?
-            ''', (featuretype,))
+        if featuretype is not None:
+            c.execute(
+                '''
+                SELECT count() FROM features
+                WHERE featuretype = ?
+                ''', (featuretype,))
+        else:
+            c.execute(
+                '''
+                SELECT count() FROM features
+                ''')
+
         results = c.fetchone()
         if results is not None:
             results = results[0]
@@ -595,6 +603,74 @@ class FeatureDB(object):
         db._update_relations()
         db._finalize()
 
+    def add_relation(self, parent, child, level, parent_func=None, child_func=None):
+        """
+        Manually add relations to the database.
+
+        Parameters
+        ----------
+        parent : str or Feature instance
+             Parent feature to add.
+
+        child : str or Feature instance
+            Child feature to add
+
+        level : int
+            Level of the relation.  For example, if parent is a gene and child
+            is an mRNA, then you might want level to be 1.  But if child is an
+            exon, then level would be 2.
+
+        parent_func, child_func : callable
+            These optional functions control how attributes are updated in the
+            database.  They both have the signature `func(parent, child)` and
+            must return a [possibly modified] Feature instance.  For example,
+            we could add the child's database id as the "child" attribute in
+            the parent::
+
+                def parent_func(parent, child):
+                    parent.attributes['child'] = child.id
+
+            and add the parent's "gene_id" as the child's "Parent" attribute::
+
+                def child_func(parent, child):
+                    child.attributes['Parent'] = parent['gene_id']
+        """
+        if isinstance(parent, basestring):
+            parent = self[parent]
+        if isinstance(child, basestring):
+            child = self[child]
+
+        c = self.conn.cursor()
+        c.execute('''
+                  INSERT INTO relations (parent, child, level)
+                  VALUES (?, ?, ?)''', (parent.id, child.id, level))
+
+        if parent_func is not None:
+            parent = parent_func(parent, child)
+            self._update(parent, c)
+        if child_func is not None:
+            child = child_func(parent, child)
+            self._update(child, c)
+
+        self.conn.commit()
+
+
+    def _update(self, feature, cursor):
+        values = [list(feature.astuple()) + [feature.id]]
+        cursor.execute(
+            constants._UPDATE, *tuple(values))
+
+    def _insert(self, feature, cursor):
+        """
+        Insert a feature into the database.
+        """
+        try:
+            cursor.execute(constants._INSERT, feature.astuple())
+        except sqlite3.ProgrammingError:
+            cursor.execute(
+                constants._INSERT, feature.astuple(self.default_encoding))
+
+
     def create_introns(self, exon_featuretype='exon',
                        grandparent_featuretype='gene', parent_featuretype=None,
                        new_featuretype='intron', merge_attributes=True):
@@ -713,11 +789,11 @@ class FeatureDB(object):
                 # The start position is outside the merged feature, so we're
                 # done with the current merged feature.  Prepare for output...
                 merged_feature = dict(
-                    chrom=feature.chrom,
+                    seqid=feature.chrom,
                     source='.',
-                    featuretype=featuretype,
+                    featuretype=feature.featuretype,
                     start=current_merged_start,
-                    stop=current_merged_stop,
+                    end=current_merged_stop,
                     score='.',
                     strand=strand,
                     frame='.',
@@ -733,11 +809,11 @@ class FeatureDB(object):
         if len(features) == 1:
             feature = features[0]
         merged_feature = dict(
-            chrom=feature.chrom,
+            seqid=feature.chrom,
             source='.',
-            featuretype=featuretype,
+            featuretype=feature.featuretype,
             start=current_merged_start,
-            stop=current_merged_stop,
+            end=current_merged_stop,
             score='.',
             strand=strand,
             frame='.',
